/* eslint-disable vue/no-reserved-component-names */
/* eslint-disable vue/multi-word-component-names */
import './assets/main.css'

import { createApp, ref } from 'vue'
import { createPinia } from 'pinia'
import axios, { Axios, AxiosError, type AxiosInstance } from 'axios'
import VueJsonPretty from 'vue-json-pretty'

import Lara from '@primeuix/themes/lara'
import Accordion from 'primevue/accordion';
import AccordionPanel from 'primevue/accordionpanel';
import AccordionHeader from 'primevue/accordionheader';
import AccordionContent from 'primevue/accordioncontent';
import Button from 'primevue/button'
import Breadcrumb from 'primevue/breadcrumb'
import Checkbox from 'primevue/checkbox'
import Chip from 'primevue/chip'
import InputText from 'primevue/inputtext'
import Menubar from 'primevue/menubar'
import Message from 'primevue/message'
import Panel from 'primevue/panel'
import ProgressBar from 'primevue/progressbar'
import PrimeVue from 'primevue/config'
import Toast from 'primevue/toast'
import ToastService from 'primevue/toastservice'
import Tooltip from 'primevue/tooltip'

import App from './App.vue'
import router from './router'
import { initWebsocket } from './socket'
import type { HatchetWorker, ApiServerInfo, WorkflowRunResult, MenuDef } from './models'

const app = createApp(App)

app.use(createPinia())
app.use(router)
app.use(PrimeVue, { theme: { preset: Lara } })
app.use(ToastService)
app.component('PrettyJson', VueJsonPretty)
app.component('Button', Button)
app.component('Accordion', Accordion)
app.component('AccordionPanel', AccordionPanel)
app.component('AccordionHeader', AccordionHeader)
app.component('AccordionContent', AccordionContent)
app.component('Breadcrumb', Breadcrumb)
app.component('Menubar', Menubar)
app.component('Checkbox', Checkbox)
app.component('Chip', Chip)
app.component('InputText', InputText)
app.component('Message', Message)
app.component('Panel', Panel)
app.component('ProgressBar', ProgressBar)
app.component('Toast', Toast)
app.directive('tooltip', Tooltip);


class HatchetInfo {
  workers?: HatchetWorker[]
  async getInfo() {
    workers.value = (await apiServer.get("/worker")).data
  }

  activeWorkers(): HatchetWorker[] {
    return workers.value.filter(w => w.status == "ACTIVE")
  }
}


export async function getServerInfo() {
  try {
    apiServerInfo.value = (await apiServer.get("/")).data
  }
  catch (err) {
    message.value = `${(<AxiosError>err).message} while loading API. Try reload page?`
  }
  app.mount('#app')
}

async function bootstrap() {
  // The settings.json file is expected at the server's base url
  try {
    apiUrl = (await axios.get("/settings.json")).data["apiUrl"]
  }
  catch (err) {
    console.error(err)
    alert("Cannot load settings.")
    return
  }
  apiServer = axios.create({
    baseURL: apiUrl,
    timeout: 10000
  })
  // Really start when apiServer is known
  initWebsocket(apiUrl)
  await getServerInfo()
  await hatchetInfo.value.getInfo()
}

export async function getWfResults(wfId: string): Promise<WorkflowRunResult[]> {
  return apiServer.get<WorkflowRunResult[]>("/wf/" + wfId + "/runs").then(resp => resp.data)
}

export const aboutKnocURL: string = "https://code.philo.ydns.eu/philorg/knoc"
export const workers = ref<HatchetWorker[]>([])
export const message = ref<string>("")
export const apiServerInfo = ref<ApiServerInfo>({})
export const hatchetInfo = ref<HatchetInfo>(new HatchetInfo())
export let apiServer: AxiosInstance
export let apiUrl: string
export const menu = ref<MenuDef[]>([])

bootstrap()
